
// Analog meter firmware for use in dual rf power meter
// Poll van der Wouw PA3BYV  09-03-2020
// Naar een idee van DC5ZM dual power meter v1, Funkamateur feb 2020 
//
// Deze software is vrijelijk te gebruiken, te modificeren of aan te vullen. Er rust geen copyright op. 
// Daar staat tegenover dat er geen garantie is dat het werkt of ergens goed voor is. 
// Gebruik op eigen risico.
//
// Hardware:
//   Arduino Pro Mini 
//   128*64 OLED screen 0.95 with SPI interface
//   two AD8318 boards
//
// necessary libraries: 
//    Adafruit GFX Library (algemene grafische functies)
//    Adafruit SSD1306 (Driver voor OLED met SSD1306)
// 
// connections:
//    OLED to A4, A5 (SDA, SCL), VCC and ground
//    AD8318 to A0 and A1 (forward power to A0, reflected power to A1), gnd and 7-12 Vdc
//    Arduino Pro Mini gnd and 7-12 Vdc 
//    Alternatively power ARDUINO via USB and AD8318 with VCC (5 V). 
//    The 5 V regulator on the AD8318 has to be shortend or removed in that case.

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

bool run_calib = false;
bool dual_meter = true;
int analogPin = A0;   // input from AD8318
int analogPiN_2 = A1; // second AD8318
int buttonPin1 = 8;
int val = 0;  // variable to store the value read
float slope, intercept;
float vswr, Return_Loss;
int frequency;
char calibration_freq[7];
char *cal_freq_str[] = {"0.9 GHz","1.9 GHz","2.2 GHz","3.6 GHz","5.8 GHz","8.0 GHz"};
float voltage_PwrMeter, voltage_PwrMeter_2;
float vRef = 5.00; // Change to actual reference voltage. Also change analogReference(DEFAULT) in setup if using EXTERNAL or INTERNAL reference
float pwr = 0.0, pwr_2 = 0.0,  pwr_mW = 0.0, pwr_mW_2 = 0.0;
int err_limit_high, err_limit_low;
int n;
int att = 0;
int cal_curve = 3; //start with calibration curve 3 (2.2GHz)
int resbox_y = 70;

// ------------------------------ S E T U P -----------------------------------------------------------

void setup(void) {
  
  //Serial.begin(9600); // start serial, only required for debugging messages 
  
  //set_calibration(3);
  pinMode(buttonPin1,INPUT_PULLUP);
  analogReference(DEFAULT); // 1.1 V reference necessitates a voltage divider 1:2 before analog 

  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {        //0x3C is SPI-address of OLED used
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }

  display.clearDisplay();
  display.setTextColor(WHITE);
  display.display();
  disp_main_screen(cal_curve, att);
  delay(2000);


} //end setup() ----------------------------------------------------------------------------------------

// --------------------------- M A I N   L O O P -------------------------------------------------------
void loop() 
{
      int i;

      analogPin = A0;
      voltage_PwrMeter = read_AD8318(analogPin);
      set_calibration(3);
      pwr = voltage_PwrMeter/slope + intercept; // main power function
      pwr_mW = pow(10,pwr/10);

      analogPin = A1;
      voltage_PwrMeter_2 = read_AD8318(analogPin);
      set_calibration(9);
      pwr_2 = voltage_PwrMeter_2/slope + intercept; // main power function
      pwr_mW_2 = pow(10,pwr_2/10);

      calculate_ReturnLoss_and_SWR();

      display_result_pwr();
      for (i=0;i<20;i++){  //check for button press while we wait
        if(button_Menu()>0){
          disp_main_screen(cal_curve, att); // re-display main screen to allert to new calibration curve
          delay(4000);
        }
        delay(100);
      }

}
// -------------------------- S U B R O U T I N E S ----------------------------------------------------
void display_result_pwr()    // displays results from power meter
{
  byte yl;
  display.clearDisplay(); // clear display
  display.setCursor(0,0);
  display.setTextSize(2);
  display.print(pwr,1);
  display.print(" ");
  display.setTextSize(1);
  display.setCursor(64,6);
  display.print("dBm");
  display.setTextSize(2);
  display.setCursor(96,0);
  display.print("P1");
  if (run_calib){
    display.setTextSize(2);
    display.setCursor(0, 16);
    display.print(voltage_PwrMeter,3);
    display.setTextSize(1);
    display.setCursor(66,22);
    display.print("V"); 
  }
  if (run_calib) yl = 34; else yl = 16;
  display.setCursor(0,yl);
  display.setTextSize(2);
  display.print(pwr_2,1);
  display.print(" ");
  display.setTextSize(1);
  display.setCursor(64,yl+6);
  display.print("dBm");
  display.setTextSize(2);
  display.setCursor(96,yl);
  display.print("P2");
  if (run_calib){
    display.setTextSize(2);
    display.setCursor(0, 50);
    display.print(voltage_PwrMeter_2,3);
    display.setTextSize(1);
    display.setCursor(66,56);
    display.print("V"); 
  }
  else{
    if(pwr >= pwr_2){
      display.setCursor(0,40);
      display.setTextSize(2);
      display.print(vswr,1);
      display.setTextSize(1);
      display.setCursor(64,40+6);
      display.println("VSWR");
      display.setTextSize(1); 
    }
    else{
      display.setCursor(0,40);
      display.setTextSize(2); 
      display.println(" P2 > P1");      
      //display.println(" Change");
      //display.println(" Channels");
    }
  }

  display.display();
}
// -----------------------------------------------------------------------------------------------------
float read_AD8318(int an_pin)                  // returns average of 10 readings from analog pin an_pin
{ 
  byte i;
  int inp_adc;                                    // result from adc stored here

  analogRead(an_pin);                             // discard first measurement
  
  inp_adc  =  0.0;                                // clear result
  for(i=0; i<20; i++) {                           // read 20 times     
      inp_adc  =  inp_adc + analogRead(an_pin);   // sum results
      delay(1);
  }
  inp_adc = inp_adc/20;                           // divide by 20 to get average
  return (float(inp_adc)+0.5)*vRef/1024;          // convert to voltage with respect to reference 
}                                                              
// -----------------------------------------------------------------------------------------------------
void disp_main_screen(int cal_curve, int att)
{
  display.clearDisplay();
  display.setCursor(0, 4);
  display.setTextColor(WHITE);
  display.setTextSize(1);
  display.println("  RF Power Meter");
  display.println(" ");

  display.setTextSize(1);
  display.println("AD8318 0.001-8 GHz");
  display.println("-55 to -5, max +5 dB");
  display.setTextSize(1);
  set_calibration(cal_curve); 
  display.println(" ");
  //display.print("Calibration ");
  //display.print(cal_freq_str[cal_curve-1]);
  if(cal_curve>0){
    display.print(cal_freq_str[cal_curve-1]);
    run_calib = false;
  }
  else {
    display.print("Volt mode");
    run_calib = true;
  }
  //display.print(" Attenuation ");
  //display.println(att);
  
  display.display();
}
// -----------------------------------------------------------------------------------------------------

void set_calibration(byte n)
{
  int cal_frequency[6] = {900,1900,2200,3600,5800,8000}; //

//  int onedb_err_limit_high[6] = {-1,-5,-4,-3,0,0};  // 1 db error limits
//  int onedb_err_limit_low[6] = {-55,-68,-60,-52,-56,-54};

  int cal_slope[6] = {-245, -244, -244, -243, -243, -300}; // in V/db * 10000 from datasheet
  int cal_intercept[6] = {220, 204, 196, 198, 250, 370}; // in dBm * 10 from datasheet
  int cal_slope_2[6] = {-245, -244, -244, -243, -243, -300}; // in V/db * 10000 from datasheet
  int cal_intercept_2[6] = {220, 204, 196, 198, 250, 370}; // in dBm * 10 from datasheet

/* debug
  Serial.print("N ");
  Serial.print(n);
  Serial.print(" :: ");
*/ //debug
  if (n>0 && n<7){
    n = n-1;
    slope = float(cal_slope[n])/10000;
    intercept = float(cal_intercept[n])/10;
  }
  else if (n>6 && n<13){
    n = n-7;
    slope = float(cal_slope_2[n])/10000;
    intercept = float(cal_intercept_2[n])/10;
  }

  else return;
  
  frequency = cal_frequency[n];
  strcpy(calibration_freq, cal_freq_str[n]); 
//  err_limit_high = onedb_err_limit_high[n];
//  err_limit_low = onedb_err_limit_low[n];

/* debug
  Serial.print(calibration_freq);
  Serial.print(" slope: ");
  Serial.print(slope);
  Serial.print(",  intercept: ");
  Serial.println(intercept);
*/ //debug
}

// -----------------------------------------------------------------------------------------------------
void calculate_ReturnLoss_and_SWR()
{   
  float RL_linear;
  
  Return_Loss = pwr - pwr_2;    
  RL_linear = pow(10,Return_Loss/20 );
  vswr = (RL_linear+1)/(RL_linear-1);
}

// -----------------------------------------------------------------------------------------------------
int button_Menu()
{   
  int i;
  bool released, longpress, shortpress;

  if(digitalRead(buttonPin1)==HIGH) return 0; // button NOT pushed
  longpress=true; // button pushed, see if it is a long press

  for(i=0;i<5;i++){         // wait for at least 2.5 s button push
    delay(500);
    released = digitalRead(buttonPin1);
    if(released == true && i<4) return 0;   // no long button press
  }
  display_CalCurve();  
  // found a longpress now wait for release
  while (longpress){
    longpress = !digitalRead(buttonPin1);
    delay(50);
  }
  // cyclic change of calibration curve with each button push
  while (true){
    display_CalCurve();  
    for(i=0;i<50;i++){ // wait 5 seconds for button press
      delay(100);
      released = digitalRead(buttonPin1);
      if(released == false) {   // found button press
        break; // break from for loop
      }
    }
    if(released == true) return 1;   // return to measurement screen after 5 seconds without button push

    for(i=0;i<9;i++){         // wait for at least 2.5 s button push for a long button press
      delay(250);
      if(digitalRead(buttonPin1)==HIGH && i<4) {
        shortpress = true;
        break;   // short button press
      }
    }
    if(!shortpress) return 1; // return to measurement screen
    shortpress = false;
    cal_curve +=1; // next calibration curve
    if (cal_curve > 6) cal_curve = 0; // cycle to first curve
  }
  
}
// -----------------------------------------------------------------------------------------------------
void display_CalCurve()
{   
  display.clearDisplay();
  display.setCursor(0, 4);
  display.setTextColor(WHITE);
  display.setTextSize(2);
  display.println("Calibrate");
  display.println(" ");
  if(cal_curve>0){
    display.println(cal_freq_str[cal_curve-1]);
    run_calib = false;
  }
  else {
    display.println("Volt mode");
    run_calib = true;
  }
  //display.println(" ");
  //display.println("Short push to change");  
  //display.println("Long to return");

  display.display();

}
